#include <stdio.h>
#include <linux/vt.h>
#include <errno.h>
#include <termio.h>
#include <signal.h>
#include <unistd.h>
#include <pwd.h>

#ifdef SHADOW
#undef SHADOW
#include <shadow.h>
#endif

#include "ansi.h"

#define MAXPASSWDLEN 128
#define ADMIN "root" /* This is the one we use for admin password */
#define DELAY 2 /* Seconds to delay between each allowed attempt */

void init(void);
void uninit(void);
void lock_vtswitch(void);
void unlock_vtswitch(void);
void echooff(void);
void echoon(void);
void credit(void);
int checkpass(char *passwd, char *user);

#define OWNER (getpwuid(getuid())->pw_name)


#ifdef SHADOW
#define get_cipher(x) getspnam(x)->sp_pwdp
#else
#define get_cipher(x) getpwnam(x)->pw_passwd
#endif

#ifdef SHADOW
#define rewind_pwd() setspent()
#else
#define rewind_pwd() setpwent()
#endif


int main(int ac, char **av)
{
  int status;
  char input[MAXPASSWDLEN];
  int count=0;

  while (1)
    {
      init();
      credit();

      printf("This TTY owned by %s\n", OWNER);
      printf("Password: ");
      fgets(input, MAXPASSWDLEN-1, stdin);
      
      if (checkpass(input, OWNER)) break;
      if (checkpass(input, ADMIN)) break;

      bzero(input, MAXPASSWDLEN); /* To not keep input in memory */
      printf("\nAccess denied.\n", count++);
      sleep(DELAY);
    }

  uninit();

  if (count) printf("There has been %i failed attempt%s to log in.\n", count, count == 1 ? "" : "s");

  return 1;
}

void lock_vtswitch(void) { if (ioctl(fileno(stdin), VT_LOCKSWITCH)) perror("ttylock"); }
void unlock_vtswitch(void) { if (ioctl(fileno(stdin), VT_UNLOCKSWITCH)) perror("ttyunlock"); }

void echooff(void)
{
  struct termios termios;

  ioctl(fileno(stdin), TCGETS, &termios);
  termios.c_lflag &=~ECHO;
  ioctl(fileno(stdin), TCSETS, &termios);
}

void echoon(void)
{
  struct termios termios;

  ioctl(fileno(stdin), TCGETS, &termios);
  termios.c_lflag |= ECHO;
  ioctl(fileno(stdin), TCSETS, &termios);
}

int checkpass(char *input, char *user)
{
  char *crypted;
  char *temp;

  rewind_pwd();
  crypted = get_cipher(user);
  
  if (!strcmp(crypted, crypt(input, crypted)))
    return 1;

  return 0;
}

void init(void)
{
  signal(SIGINT, SIG_IGN);
  signal(SIGQUIT, SIG_IGN);
  signal(SIGTSTP, SIG_IGN);
  
  lock_vtswitch();
  echooff();

  cursor_pos(0,0);
  clrscr();
}

void uninit(void)
{
  unlock_vtswitch();
  echoon();
  cursor_pos(0,0);
  clrscr();
}

void credit(void)
{
  printf("Console locker, v.%s, by Martin.C <flipflop@swipnet.se>\n", VERSION);
  printf("--------------\n");
}
